@@ -31,7 +31,7 @@ else:
         exec("""exec _code_ in _globs_, _locs_""")
 
 from functools import partial
-from inspect import getargspec, ismethod, isclass, formatargspec
+from inspect import ismethod, isclass, formatargspec
 from collections import namedtuple
 from threading import Lock, RLock
 
@@ -99,11 +99,6 @@ class _AdapterFunctionSurrogate(CallableObjectProxy):
         if 'signature' not in globals():
             return self._self_adapter.__signature__
         else:
-            # Can't allow this to fail on Python 3 else it falls
-            # through to using __wrapped__, but that will be the
-            # wrong function we want to derive the signature
-            # from. Thus generate the signature ourselves.
-
             return signature(self._self_adapter)
 
     if PY2:
@@ -117,6 +112,13 @@ class _BoundAdapterWrapper(BoundFunctionWrapper):
         return _AdapterFunctionSurrogate(self.__wrapped__.__func__,
                 self._self_parent._self_adapter)
 
+    @property
+    def __signature__(self):
+        if 'signature' not in globals():
+            return self.__wrapped__.__signature__
+        else:
+            return signature(self._self_parent._self_adapter)
+
     if PY2:
         im_func = __func__
 
@@ -181,17 +183,17 @@ def decorator(wrapper=None, enabled=None, adapter=None):
     # decorator. In that case parts of the function '__code__' and
     # '__defaults__' attributes are used from the adapter function
     # rather than those of the wrapped function. This allows for the
-    # argument specification from inspect.getargspec() to be overridden
-    # with a prototype for a different function than what was wrapped.
-    # The 'enabled' argument provides a way to enable/disable the use
-    # of the decorator. If the type of 'enabled' is a boolean, then it
-    # is evaluated immediately and the wrapper not even applied if
-    # it is False. If not a boolean, it will be evaluated when the
-    # wrapper is called for an unbound wrapper, and when binding occurs
-    # for a bound wrapper. When being evaluated, if 'enabled' is callable
-    # it will be called to obtain the value to be checked. If False,
-    # the wrapper will not be called and instead the original wrapped
-    # function will be called directly instead.
+    # argument specification from inspect.getargspec() and similar
+    # functions to be overridden with a prototype for a different
+    # function than what was wrapped. The 'enabled' argument provides a
+    # way to enable/disable the use of the decorator. If the type of
+    # 'enabled' is a boolean, then it is evaluated immediately and the
+    # wrapper not even applied if it is False. If not a boolean, it will
+    # be evaluated when the wrapper is called for an unbound wrapper,
+    # and when binding occurs for a bound wrapper. When being evaluated,
+    # if 'enabled' is callable it will be called to obtain the value to
+    # be checked. If False, the wrapper will not be called and instead
+    # the original wrapped function will be called directly instead.
 
     if wrapper is not None:
         # Helper function for creating wrapper of the appropriate
@@ -206,7 +208,7 @@ def decorator(wrapper=None, enabled=None, adapter=None):
                     ns = {}
                     if not isinstance(adapter, string_types):
                         adapter = formatargspec(*adapter)
-                    exec_('def adapter{0}: pass'.format(adapter), ns, ns)
+                    exec_('def adapter{}: pass'.format(adapter), ns, ns)
                     adapter = ns['adapter']
 
                 return AdapterWrapper(wrapped=wrapped, wrapper=wrapper,
@@ -255,7 +257,7 @@ def decorator(wrapper=None, enabled=None, adapter=None):
                     # we need to first check that use of the decorator
                     # hadn't been disabled by a simple boolean. If it was,
                     # the target function to be wrapped is returned instead.
-                    
+
                     _enabled = enabled
                     if type(_enabled) is bool:
                         if not _enabled:
@@ -393,9 +395,12 @@ def decorator(wrapper=None, enabled=None, adapter=None):
 
         # We first return our magic function wrapper here so we can
         # determine in what context the decorator factory was used. In
-        # other words, it is itself a universal decorator.
+        # other words, it is itself a universal decorator. The decorator
+        # function is used as the adapter so that linters see a signature
+        # corresponding to the decorator and not the wrapper it is being
+        # applied to.
 
-        return _build(wrapper, _wrapper)
+        return _build(wrapper, _wrapper, adapter=decorator)
 
     else:
         # The wrapper still has not been provided, so we are just
@@ -424,7 +429,7 @@ def synchronized(wrapped):
 
     if hasattr(wrapped, 'acquire') and hasattr(wrapped, 'release'):
         # We remember what the original lock is and then return a new
-        # decorator which acceses and locks it. When returning the new
+        # decorator which accesses and locks it. When returning the new
         # decorator we wrap it with an object proxy so we can override
         # the context manager methods in case it is being used to wrap
         # synchronized statements with a 'with' statement.
@@ -453,7 +458,7 @@ def synchronized(wrapped):
     # Following only apply when the lock is being created automatically
     # based on the context of what was supplied. In this case we supply
     # a final decorator, but need to use FunctionWrapper directly as we
-    # want to derive from it to add context manager methods in in case it is
+    # want to derive from it to add context manager methods in case it is
     # being used to wrap synchronized statements with a 'with' statement.
 
     def _synchronized_lock(context):
@@ -474,10 +479,7 @@ def synchronized(wrapped):
             # creation and assignment of the lock attribute against
             # the context.
 
-            meta_lock = vars(synchronized).setdefault(
-                    '_synchronized_meta_lock', Lock())
-
-            with meta_lock:
+            with synchronized._synchronized_meta_lock:
                 # We need to check again for whether the lock we want
                 # exists in case two threads were trying to create it
                 # at the same time and were competing to create the
@@ -496,7 +498,7 @@ def synchronized(wrapped):
         # desired context is held. If instance is None then the
         # wrapped function is used as the context.
 
-        with _synchronized_lock(instance or wrapped):
+        with _synchronized_lock(instance if instance is not None else wrapped):
             return wrapped(*args, **kwargs)
 
     class _FinalDecorator(FunctionWrapper):
@@ -510,3 +512,5 @@ def synchronized(wrapped):
             self._self_lock.release()
 
     return _FinalDecorator(wrapped=wrapped, wrapper=_synchronized_wrapper)
+
+synchronized._synchronized_meta_lock = Lock()
